<?php
namespace SalvatoreEckel\T3wrapper\Controller;

/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Salvatore Eckel <salvaracer@gmx.de>, ie
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * WrapperController
 */
class WrapperController extends AbstractController {

    // TypoScript settings
    protected $settings = array();

    // id of selected page
    protected $id;

    // info of selected page
    protected $pageinfo;
 
    protected function initializeAction() {
        $this->id = (int)\TYPO3\CMS\Core\Utility\GeneralUtility::_GP('id');
        $this->pageinfo = \TYPO3\CMS\Backend\Utility\BackendUtility::readPageAccess($this->id, $GLOBALS['BE_USER']->getPagePermsClause(1));
 
        $configurationManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\BackendConfigurationManager');
 
        $this->settings = $configurationManager->getConfiguration(
            $this->request->getControllerExtensionName(),
            $this->request->getPluginName()
        );
    }

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
		$this->updateConfig();

		$wrappers = $this->wrapperRepository->findAll();
		$this->view->assign('wrappers', $wrappers);
	}

	/**
	 * action generators
	 *
	 * @return void
	 */
	public function generatorsAction() {
	}

	/**
	 * action index
	 *
	 * @return void
	 */
	public function indexAction() {
		
	}

	/**
	 * action new
	 *
	 * @return void
	 */
	public function newAction() {
		$tree = $this->getPageTreeView();
		$this->view->assign('tree',$tree->tree);
	}

	/**
	 * action create
	 *
	 * @param \SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $newWrapper
	 * @return void
	 */
	public function createAction(\SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $newWrapper) {
		$this->addFlashMessage('Your T3wrapper object was created.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
		$this->wrapperRepository->add($newWrapper);

		$this->persistenceManager->persistAll();

		$result = $this->wrapperRepository->findLastCreated();
		$wrapper = $result->getFirst();

		$this->updateConfig();

		$this->redirect('edit',NULL,NULL,array('wrapper' => $wrapper));
	}

	/**
	 * action edit
	 *
	 * @param \SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $wrapper
	 * @ignorevalidation $wrapper
	 * @return void
	 */
	public function editAction(\SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $wrapper) {
		$tree = $this->getPageTreeView();

		$this->view->assign('tree',$tree->tree);
		$this->view->assign('wrapper', $wrapper);
	}

	/**
	 * action update
	 *
	 * @param \SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $wrapper
	 * @return void
	 */
	public function updateAction(\SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $wrapper) {
		$this->addFlashMessage('The object was updated.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
		$this->wrapperRepository->update($wrapper);

		$this->persistenceManager->persistAll();

		$this->updateConfig();

		$this->redirect('edit',NULL,NULL,array('wrapper' => $wrapper));
	}

	/**
	 * action delete
	 *
	 * @param \SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $wrapper
	 * @return void
	 */
	public function deleteAction(\SalvatoreEckel\T3wrapper\Domain\Model\Wrapper $wrapper) {
		$this->addFlashMessage('The object was deleted.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
		$this->wrapperRepository->remove($wrapper);
		$this->redirect('list');
	}

	/**
	 * action updateConfig
	 *
	 * reads the active wrappers and write an updated typoscript configuration
	 *
	 * @return void
	 */
	protected function updateConfig() {
		// Paths
		$typoscriptPath = __DIR__ . '/../../Configuration/TypoScript/';
		$typenumTemplatePath = $typoscriptPath . 'typenum.tmpl';
		$includesTsFile = $typoscriptPath . 'includes.txt';

		$output = array();

		if (file_exists($typenumTemplatePath)) {
			$wrappers = $this->wrapperRepository->findAll();
			foreach ($wrappers as $key => $wrapper) {
				$typenumTemplate = file_get_contents($typenumTemplatePath);

				$placeholderArray = array(
					"uid"=>$wrapper->getUid(),
					"typeNum"=>$wrapper->getTypeNum(),
					"disableAllHeaderCode"=>$wrapper->getDisableAllHeaderCode(),
					"admPanel"=>$wrapper->getAdmPanel(),
					"additionalHeaders"=>$wrapper->getAdditionalHeaders(),
					"extensionName"=>$wrapper->getExtensionName(),
					"controller"=>$wrapper->getController(),
					"pluginName"=>$wrapper->getPluginName(),
					"vendorName"=>$wrapper->getVendorName(),
					"switchableControllerActions"=>$wrapper->getSwitchableControllerActions() ? $wrapper->getSwitchableControllerActions() : '1 = index',
					"abstractSettings"=>$wrapper->getAbstractSettings() ? $wrapper->getAbstractSettings() : 'storagePid = 0'
				);

				foreach($placeholderArray as $placeholder => $value) {
					$typenumTemplate = str_replace('###' . $placeholder . '###', $value, $typenumTemplate);
				}

				$output[] = $typenumTemplate;
			}

			$filcontent = '';

			foreach ($output as $key => $value) {
				$filecontent .= $value;
			}

			file_put_contents($includesTsFile, $filecontent);
			$this->addFlashMessage('Wrapper Configurations were updated.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		} else {
			$this->addFlashMessage('TYPENUM TEMPLATE NOT FOUND.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		}
	}

	/**
	 * action recycle
	 *
	 * @return void
	 */
	public function recycleAction() {
	}
}